using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SampleClient.PTBurnJobsCtrl
{
    public partial class PTBurnJobsCtrl : UserControl
    {
        private string Robot;
        private Robot CurrentRobot;
        private StatusFile MyStatusFile;
        private string Server;
        private string StatusFile;
        public JobFile myJob;
        
        public PTBurnJobsCtrl()
        {
            InitializeComponent();
            myJob = new JobFile();
        }

        public void SetRobot(Robot CurrentRobotIn, string sServerIn)
        {
            if (CurrentRobotIn == null)
            {
                groupBoxAddData.Enabled = false;
                groupBoxAddLabel.Enabled = false;
                groupBoxSetOptions.Enabled = false;
                this.comboBoxDriveUsage.Visible = false;
                this.labelDriveUsage.Visible = false;
                this.comboBoxBinUsage.Visible = false;
                this.labelBinUsage.Visible = false;
            }
            else
            {
                groupBoxAddData.Enabled = true;
                groupBoxAddLabel.Enabled = true;
                groupBoxSetOptions.Enabled = true;               
                CurrentRobot = CurrentRobotIn;
                this.Robot = CurrentRobot.Name;
                this.Server = sServerIn;
                this.StatusFile = "\\\\" + this.Server + "\\PTBurnJobs\\Status\\" + CurrentRobot.StatusFile;
                MyStatusFile = new StatusFile(this.StatusFile);
                if (CurrentRobot.NumDrives > 1)
                {
                    this.comboBoxDriveUsage.Visible = true;
                    this.labelDriveUsage.Visible = true;
                }
                if (CurrentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_SE)
                {
                    this.comboBoxBinUsage.Visible = false;
                    this.labelBinUsage.Visible = false;
                }
                else
                {
                    this.comboBoxBinUsage.Visible = true;
                    this.labelBinUsage.Visible = true;
                }
            }
        }

        private void listView1_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void listView1_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);


            // loop through the string array, adding each filename to the ListBox
            foreach (string file in files)
            {
                string[] item = {file, "\\" + Path.GetFileName(file)};
                ListViewItem myItem = new ListViewItem(item);
                this.listView1.Items.Add(myItem);
                this.labelDragFiles.Visible = false;
            }
        }

        private void buttonPrintFileBrowse_Click(object sender, EventArgs e)
        {
            this.openFileDialog1.ShowDialog();
            if (!String.IsNullOrEmpty(this.openFileDialog1.FileName))
            {
                this.comboBoxPrintFile.Items.RemoveAt(0);
                this.comboBoxPrintFile.Items.Insert(0, this.openFileDialog1.FileName);
                this.comboBoxPrintFile.SelectedIndex = 0;
                UpdatePreview();
            }
        }

        private void buttonSaveAsTest_Click(object sender, EventArgs e)
        {
            this.SendJob("c:\\PTBurnTests\\");
        }

        private void buttonGo_Click(object sender, EventArgs e)
        {
            bool fJobAlreadyFound = false;
            DirectoryInfo myDir = new DirectoryInfo("\\\\" + this.Server + "\\PTBurnJobs\\");
            foreach (DirectoryInfo lowDir in myDir.GetDirectories())
            {
                if (lowDir.Name.EndsWith(textBoxJobName.Text, true, null))
                {
                    fJobAlreadyFound = true;
                    break;
                }
            }
            if (fJobAlreadyFound)
            {
                foreach (FileInfo myFile in myDir.GetFiles())
                {
                    if (myFile.Name.EndsWith(textBoxJobName.Text, true, null))
                    {
                        fJobAlreadyFound = true;
                        break;
                    }
                }
            }

            if (fJobAlreadyFound)
                MessageBox.Show("Please change the name of the job and try again");
            else
                this.SendJob("\\\\" + this.Server + "\\PTBurnJobs\\");
        }

        private void PTBurnJobsCtrl_Load(object sender, EventArgs e)
        {
            this.comboBoxDriveUsage.Items.Add("Use Any Drive");
            this.comboBoxDriveUsage.Items.Add("Use Top Drive Only");
            this.comboBoxDriveUsage.Items.Add("Use Bottom Drive Only");
            this.comboBoxDriveUsage.SelectedIndex = 0;

            this.comboBoxBinUsage.Items.Add("Don't Care");
            this.comboBoxBinUsage.Items.Add("Left Bin");
            this.comboBoxBinUsage.Items.Add("Right Bin");
            this.comboBoxBinUsage.Items.Add("Use Old Bin Selection");
            this.comboBoxBinUsage.SelectedIndex = 0;

            this.comboBoxJobTypes.Items.Add("Data Job");
            this.comboBoxJobTypes.Items.Add("Audio Job");
            this.comboBoxJobTypes.Items.Add("Image Job");
            this.comboBoxJobTypes.Items.Add("Read Job");
            this.comboBoxJobTypes.SelectedIndex = 0;

            this.comboBoxReadFormat.Items.Add("ReadGI");
            this.comboBoxReadFormat.Items.Add("ReadData");
            this.comboBoxReadFormat.SelectedIndex = 0;

            this.checkBoxCreateSubFolders.Checked = true;
            this.textBoxReadTo.Text = "";


            if (Properties.Settings.Default.UnicodeEncoding)
                this.checkBoxUnicode.CheckState = CheckState.Checked;
            else
                this.checkBoxUnicode.CheckState = CheckState.Unchecked;



            this.numericUpDownCopies.Value = 1;

            this.comboBoxPrintFile.Items.Insert(0, "");
            this.comboBoxPrintFile.Items.Insert(1, "Merge Example");
            this.comboBoxPrintFile.SelectedIndex = 0;



            this.textBoxJobName.Text = "test";
        }

        private void SendJob(string Folder)
        {
            myJob.Copies = (uint)numericUpDownCopies.Value;
            myJob.Importance = 4;
            myJob.JobID = textBoxJobName.Text;
            myJob.ClientID = Environment.MachineName;
            myJob.VolumeName = textBoxVolume.Text;

            myJob.Data.Clear();

            if (checkBoxUseAnyRobot.Checked)
                myJob.Robot = "";
            else
                myJob.Robot = CurrentRobot.Name;

            if (comboBoxDriveUsage.SelectedIndex == 0)
                myJob.UseAnyDrive = true;
            else
            {
                myJob.UseAnyDrive = false;
                foreach (Drive myDrv in MyStatusFile.Drives)
                {
                    if (comboBoxDriveUsage.SelectedIndex == 1)
                    {
                        if (myDrv.Location.Contains("Top"))
                        {
                            myJob.DriveID = (uint)myDrv.Index;
                            break;
                        }
                    }
                    else if (comboBoxDriveUsage.SelectedIndex == 2)
                    {
                        if (myDrv.Location.Contains("Bottom"))
                        {
                            myJob.DriveID = (uint)myDrv.Index;
                            break;
                        }
                    }
                }
            }


            
            
            
            string jobName = myJob.ClientID + "_" + myJob.JobID;
            myJob.BinID = (uint)comboBoxBinUsage.SelectedIndex;
            string jobFile;
            int nJobIndex = 1;
            while (true)
            {
                if (checkBoxSimulate.Checked)
                    jobFile = Folder + jobName + "._rq";
                else
                    jobFile = Folder + jobName + ".jrq";
                if (File.Exists(jobFile))
                {
                    jobName = myJob.ClientID + "_" + myJob.JobID + "(" + nJobIndex.ToString() + ")";
                    nJobIndex++;
                }
                else
                {
                    break;
                }
            }


            string jobDirectory = Folder + jobName;
            string dataDirectory = Folder + jobName + "\\Data";
            string printDirectory = Folder + jobName + "\\Print";

            if (!Directory.Exists(jobDirectory))
            {
                Directory.CreateDirectory(jobDirectory);

                if (!String.IsNullOrEmpty(comboBoxPrintFile.Items[0].ToString()) ||
                    comboBoxPrintFile.SelectedIndex == 1)
                {
                    Directory.CreateDirectory(printDirectory);
                    string sPrintFileFrom;

                    if (comboBoxPrintFile.SelectedIndex == 0)
                    {
                        sPrintFileFrom = comboBoxPrintFile.Items[0].ToString();
                        myJob.PrintLabel = printDirectory + "\\" + Path.GetFileName(comboBoxPrintFile.Items[0].ToString());
                    }
                    else
                    {
                        sPrintFileFrom = Path.GetDirectoryName(Application.ExecutablePath) + "\\MergeExample.std";
                        myJob.PrintLabel = printDirectory + "\\MergeExample.std";
                        if (File.Exists(textBoxMergePicture.Text))
                        {
                            myJob.MergePicture = printDirectory + "\\" + Path.GetFileName(textBoxMergePicture.Text);
                            File.Copy(textBoxMergePicture.Text, myJob.MergePicture);
                        }
                        else
                            myJob.MergePicture = "";
                        myJob.MergeTitle = textBoxMergeTitle.Text;
                    }
                    try
                    {
                        File.Copy(sPrintFileFrom, myJob.PrintLabel);
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(e.Message);
                    }
                }
                if (this.listView1.Items.Count > 0)
                {
                    Directory.CreateDirectory(dataDirectory);
                    for (int i = 0; i < listView1.Items.Count; i++)
                    {
                        DataEntry myEntry = new DataEntry();
                        myEntry.File = dataDirectory + "\\" + Path.GetFileName(listView1.Items[i].SubItems[0].Text);
                        myEntry.FileTo = listView1.Items[i].SubItems[1].Text;
                        myEntry.Session = 0;
                        if (this.comboBoxJobTypes.SelectedIndex == 1)
                        {
                            myEntry.DataType = DataFileType.AUDIO;
                            myEntry.SetPreGap = false;
                        }
                        else if (this.comboBoxJobTypes.SelectedIndex == 2)
                        {
                            myEntry.DataType = DataFileType.IMAGE;
                        }
                        else
                        {
                            myEntry.DataType = DataFileType.DATA;
                        }

                        myJob.Data.Add(myEntry);


                        string entry = listView1.Items[i].SubItems[0].Text;
                        if (File.Exists(entry))
                        {
                            File.Copy(entry, dataDirectory + "\\" + Path.GetFileName(entry));
                        }
                        else
                        {
                            if (Directory.Exists(entry))
                            {
                                DirectoryInfo src = new DirectoryInfo(entry);
                                DirectoryInfo dest = new DirectoryInfo(dataDirectory + "\\" + Path.GetFileName(entry));
                                copyDirectory(src, dest, true);
                            }
                        }
                    }
                }
                if (comboBoxJobTypes.SelectedIndex == 3)
                {
                    myJob.ReadDataFormat = this.comboBoxReadFormat.SelectedItem.ToString();
                    myJob.ReadDataTo = this.textBoxReadTo.Text;
                    myJob.CreateSubFolders = this.checkBoxCreateSubFolders.Checked;
                    myJob.SendReadJob(jobFile);
                }
                else
                    myJob.SendJobFile(jobFile);
            }
        }

        public static void copyDirectory(DirectoryInfo srcPath, DirectoryInfo destPath, bool recursive)
        {
            if (!destPath.Exists)
                destPath.Create();

            // copy files 
            foreach (FileInfo fi in srcPath.GetFiles())
            {
                fi.CopyTo(Path.Combine(destPath.FullName, fi.Name), true);
            }

            // copy directories 
            if (recursive)
            {
                foreach (DirectoryInfo di in srcPath.GetDirectories())
                {
                    copyDirectory(di, new DirectoryInfo(Path.Combine(destPath.FullName, di.Name)), recursive);
                }
            }
        }

        private void listView1_KeyPress(object sender, KeyPressEventArgs e)
        {

        }

        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
               
            }
        }

        private void comboBoxPrintFile_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);


            // loop through the string array, adding each filename to the ListBox
            foreach (string file in files)
            {
                this.comboBoxPrintFile.Items.RemoveAt(0);
                this.comboBoxPrintFile.Items.Insert(0, file);
                this.comboBoxPrintFile.SelectedIndex = 0;
                //UpdatePreview();
                break;
            }
        }

        private void comboBoxPrintFile_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void UpdatePreview()
        {
            string msgFile, previewFile = null;

            if (!String.IsNullOrEmpty(comboBoxPrintFile.Items[comboBoxPrintFile.SelectedIndex].ToString()))
            {
                if (File.Exists(comboBoxPrintFile.Items[comboBoxPrintFile.SelectedIndex].ToString()))
                {
                    msgFile = "\\\\" + this.Server + "\\PTBurnJobs\\GenPreview.ptm";
                    if (!File.Exists(msgFile))
                    {
                        TextWriter tw;
                        if( Properties.Settings.Default.UnicodeEncoding)
                            tw = new StreamWriter(msgFile, false, Encoding.Unicode);
                        else
                            tw = new StreamWriter(msgFile);

                        previewFile = Path.GetDirectoryName(Application.ExecutablePath) + "\\PrintPreview.jpg";
                        if (File.Exists(previewFile))
                            File.Delete(previewFile);
                        if (tw != null)
                        {
                            tw.WriteLine("Message=GENERATE_PREVIEW");
                            if (comboBoxPrintFile.SelectedIndex == 0)
                                tw.WriteLine("PathFrom=" + comboBoxPrintFile.Items[0].ToString());
                            else
                                tw.WriteLine("PathFrom=" + Path.GetDirectoryName(Application.ExecutablePath) + "MergeExample.std");

                            tw.WriteLine("PathTo=" + previewFile);
                            tw.WriteLine("DPI=72");
                            tw.Close();
                        }
                    }


                    bool fPreview = UtilityFunctions.UtilityFunctions.WaitUntilExists(previewFile, 10000);

                    if (fPreview)
                    {
                        
                        this.panel1.BackgroundImage = Image.FromFile(previewFile);
                    }
                }
            }
        }

        private void comboBoxPrintFile_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBoxPrintFile.SelectedIndex == 1)
                this.groupBoxMergeSettings.Visible = true;
            else
                this.groupBoxMergeSettings.Visible = false;
        }

        private void comboBoxPrintFile_TextUpdate(object sender, EventArgs e)
        {

        }

        private void listView1_Click(object sender, EventArgs e)
        {
           
        }

        private void listView1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                if (listView1.SelectedItems.Count > 0)
                {
                    FormChangePath myChangePath = new FormChangePath(this.listView1.SelectedItems[0].SubItems[1].Text);
                    DialogResult myRst = myChangePath.ShowDialog();
                    if (myRst == DialogResult.OK)
                    {
                        this.listView1.SelectedItems[0].SubItems[1].Text = myChangePath.PathTo;
                    }
                }
            }
        }

        private void buttonClearList_Click(object sender, EventArgs e)
        {
            this.listView1.Items.Clear();
        }

        private void buttonBrowseMergePicture_Click(object sender, EventArgs e)
        {
            if (DialogResult.OK == this.openFileDialogImage.ShowDialog())
            {
                this.textBoxMergePicture.Text = openFileDialogImage.FileName;
            }
        }

        private void buttonMoreOptions_Click(object sender, EventArgs e)
        {
            FormOptions options = new FormOptions(this.CurrentRobot, this.myJob);
            options.ShowDialog();
        }

        private void comboBoxJobTypes_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.comboBoxJobTypes.SelectedIndex == 3)
            {
                this.groupBoxAddData.Visible = false;
                this.groupBoxAddLabel.Visible = false;
                this.groupBoxReadJob.Visible = true;
            }
            else
            {
                this.groupBoxAddData.Visible = true;
                this.groupBoxAddLabel.Visible = true;
                this.groupBoxReadJob.Visible = false;
            }
        }

        private void buttonClearPrint_Click(object sender, EventArgs e)
        {
            this.comboBoxPrintFile.Items.RemoveAt(0);
            this.comboBoxPrintFile.Items.Insert(0, "");
        }

        private void checkBoxUnicode_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxUnicode.CheckState == CheckState.Checked)
                Properties.Settings.Default.UnicodeEncoding = true;
            else
                Properties.Settings.Default.UnicodeEncoding = false;

            Properties.Settings.Default.Save();
        }

    }
}
