using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SampleClient.PTBurnStatusCtrl
{
    public partial class PTBurnStatusCtrl : UserControl
    {
        Robot CurrentRobot;
        private StatusFile myStatusFile;
        private string Server;
        private string StatusFile;
        
        public PTBurnStatusCtrl()
        {
            InitializeComponent();
        }

        public void SetRobot(Robot currentRobotIn, string serverIn)
        {
            if (currentRobotIn == null)
            {
                groupBox3.Enabled = false;
                groupBox4.Enabled = false;
                this.pictureBoxRobot.Image = null;
                this.labelRobotName.Text = "None";
                this.labelRobotDetails.Text = "";
            }
            else
            {
                groupBox3.Enabled = true;
                groupBox4.Enabled = true;               
                CurrentRobot = currentRobotIn;
                this.Server = serverIn;
                this.StatusFile = "\\\\" + this.Server + "\\PTBurnJobs\\Status\\" + CurrentRobot.StatusFile;
                myStatusFile = new StatusFile(this.StatusFile);

                string file;
                if (CurrentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHERII)
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPII.png";
                else if (CurrentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_SE)
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPSE.png";
                else if (CurrentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHER_XRP)
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPXRP.png";
                else if (CurrentRobot.RobotType == RobotType.ROBOT_DISCPUBLISHERPRO)
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPPRO.png";
                else if (CurrentRobot.RobotType == RobotType.ROBOT_RACKMOUNT_DPII)
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\DPXR.png";
                else
                    file = Path.GetDirectoryName(Application.ExecutablePath) + "\\Images\\NoneFound.png";

                if (File.Exists(file))
                {
                    Image robotImage = Image.FromFile(file);

                    this.pictureBoxRobot.Image = robotImage;
                }

                this.listViewJobs.Items.Clear();
                this.myStatusFile.Refresh();
                this.UpdateJobList();
                this.UpdateRobotUI();
            }

        }

        private void listViewJobs_SelectedIndexChanged(object sender, EventArgs e)
        {
             UpdateSelectedJob();
        }

        private void UpdateDriveUI(Drive myDrive, ProgressBar myBar, Label myDetails)
        {
            if (myDrive.State == DriveState.RECORDING)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Recording";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.READING)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Reading";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.VERIFYING)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Verifying";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.DISC_LOADED)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Drive Loaded";
                myBar.Value = 0;
            }
            else if (myDrive.State == DriveState.VEFIFY_FAILED)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Verify Failed";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.VERIFY_COMPLETE)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Verify Complete";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.RECORD_FAILED)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Record Failed";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else if (myDrive.State == DriveState.RECORD_COMPLETE)
            {
                myDetails.Text = "Job " + myDrive.Job + " Disc " + myDrive.Disc;
                myBar.Text = "Record Complete";
                myBar.Value = Convert.ToInt32(myDrive.Percent);
            }
            else
            {
                myDetails.Text = "";
                myBar.Text = "Idle";
                myBar.Value = 0;
            }
        }

        private void UpdateRobotUI()
        {
            //myStatusFile.Refresh();

            if (myStatusFile != null)
            {
                foreach (Drive drv in myStatusFile.Drives)
                {
                    drv.Refresh();
                }


                if (myStatusFile.Drives.Count >= 1)
                {
                    Drive myDrive = myStatusFile.Drives[0] as Drive;
                    this.labelDriveDesc1.Text = myDrive.Description + " " + myDrive.Location;
                    UpdateDriveUI(myDrive, this.progressBarDrive1, this.labelDriveDetails1);
                    this.labelDriveDetails1.Show();
                    this.labelDriveDesc1.Show();
                    this.progressBarDrive1.Show();
                }
                else
                {
                    this.labelDriveDetails1.Hide();
                    this.labelDriveDesc1.Hide();
                    this.progressBarDrive1.Hide();
                }

                if (myStatusFile.Drives.Count >= 2)
                {
                    Drive myDrive = myStatusFile.Drives[1] as Drive;
                    this.labelDriveDesc2.Text = myDrive.Description + " " + myDrive.Location;
                    UpdateDriveUI(myDrive, this.progressBarDrive2, this.labelDriveDetails2);
                    this.labelDriveDetails2.Show();
                    this.labelDriveDesc2.Show();
                    this.progressBarDrive2.Show();
                }
                else
                {
                    this.labelDriveDetails2.Hide();
                    this.labelDriveDesc2.Hide();
                    this.progressBarDrive2.Hide();
                }
            }


            if (CurrentRobot != null)
            {
                CurrentRobot.Refresh();
               
                this.labelRobotName.Text = CurrentRobot.Name;
                this.labelRobotDetails.Text = "Robot FW: " + myStatusFile.RobotFW + "\n" +
                    "Robot Serial Num: " + myStatusFile.SerialNum + "\n" +
                    "Robot Date Manf: " + myStatusFile.ManfDate + "\n" +
                    "Status: " + CurrentRobot.Status + "\n" +
                    "Error Code: " + CurrentRobot.Error.ToString() + "\n" +
                    CurrentRobot.NumJobs + " Jobs in the System";

                UpdateSelectedJob();
            }


        }

        private void buttonSendJobAction_Click(object sender, EventArgs e)
        {
            string jobName;
            if (this.listViewJobs.SelectedItems.Count >0)
                jobName = Path.GetFileNameWithoutExtension(this.listViewJobs.SelectedItems[0].Text);
            else
                jobName = Path.GetFileNameWithoutExtension(this.listViewJobs.Items[0].Text);
            string jobFile;

            jobFile = "\\\\" + this.Server + "\\PTBurnJobs\\" + jobName + ".ptm";
            if (!File.Exists(jobFile))
            {
                TextWriter tw;
                if( Properties.Settings.Default.UnicodeEncoding)
                    tw = new StreamWriter(jobFile, false, Encoding.Unicode);
                else
                    tw = new StreamWriter(jobFile);

                if (tw != null)
                {
                    if (this.comboBoxJobActions.SelectedIndex == 0)
                    {
                        tw.WriteLine("Message=ABORT");
                        tw.WriteLine("ClientID=Administrator");
                        tw.WriteLine("RobotName=" + this.CurrentRobot.Name);
                    }
                    else if (comboBoxJobActions.SelectedIndex == 1)
                    {
                        tw.WriteLine("Message=PROCESS_DISC");
                        tw.WriteLine("DiscID=0");
                        tw.WriteLine("RobotName=" + this.CurrentRobot.Name);
                    }
                    else if (comboBoxJobActions.SelectedIndex == 2)
                    {
                        tw.WriteLine("Message=REJECT_DISC");
                        tw.WriteLine("DiscID=0");
                        tw.WriteLine("RobotName=" + this.CurrentRobot.Name);
                    }
                    else if (comboBoxJobActions.SelectedIndex == 3)
                    {
                        tw.WriteLine("Message=UNLOAD_DISC");
                        tw.WriteLine("DiscID=0");
                        tw.WriteLine("RobotName=" + this.CurrentRobot.Name);
                    }

                    tw.Close();
                }
            }
        }
        private void UpdateSelectedJob()
        {
            if (listViewJobs.SelectedItems.Count > 0)
            {
                foreach (Job myJob in myStatusFile.Jobs)
                {
                    if (myJob.Name == listViewJobs.SelectedItems[0].Text)
                    {

                        myJob.Refresh();
                        if (myJob.DiscsRemaining > 0)
                        {
                            this.labelJobDiscs.Text = myJob.DiscsRemaining.ToString() + " Discs Remaining " + myJob.GoodDiscs.ToString() + " Good Discs created " + myJob.BadDiscs.ToString() + " Bad Discs created ";
                            this.labelJobTimes.Text = "Time Started: " + myJob.TimeStarted;
                        }
                        else
                        {
                            this.labelJobDiscs.Text = myJob.GoodDiscs.ToString() + " Good Discs created " + myJob.BadDiscs.ToString() + " Bad Discs created ";
                            this.labelJobTimes.Text = "Time Started: " + myJob.TimeStarted + " Time Completed: " + myJob.TimeCompleted;
                        }
                        this.labelJobStatus.Text = myJob.Status;
                    }
                }
            }
        }


        private void UpdateJobList()
        {
            string job;
            Job myJobOnList = null;
            if (this.myStatusFile != null)
            {
                this.myStatusFile.RefreshJobs();
                //Remove any jobs from the listview that are not in the list anymore
                if (this.listViewJobs.Items.Count > 0)
                {
                    for (int i = this.listViewJobs.Items.Count - 1; i >= 0; i--)
                    {
                        job = this.listViewJobs.Items[i].SubItems[0].Text;
                        if (!myStatusFile.IsJobOnList(job, ref myJobOnList))
                        {
                            this.listViewJobs.Items[i].Remove();
                        }
                    }
                }

                //Add any new items
                foreach (Job myJob in myStatusFile.Jobs)
                {
                    if (!IsJobOnListview(myJob.Name))
                    {
                        ListViewItem myItem = new ListViewItem(myJob.Name);
                        if (myJob.CompletedJob)
                            myItem.ForeColor = Color.Blue;
                        else
                            myItem.ForeColor = Color.Green;
                        this.listViewJobs.Items.Add(myItem);
                    }
                }
            }
        }

        private bool IsJobOnListview(string sJob)
        {
            for (int i = 0; i < listViewJobs.Items.Count; i++)
            {
                if (listViewJobs.Items[i].Text == sJob)
                    return true;
            }
            return false;
        }

        private void PTBurnStatusCtrl_Load(object sender, EventArgs e)
        {
            this.comboBoxJobActions.Items.Add("Abort");
            this.comboBoxJobActions.Items.Add("Process Disc");
            this.comboBoxJobActions.Items.Add("Reject Disc");
            this.comboBoxJobActions.Items.Add("Unload Disc");
            this.comboBoxJobActions.SelectedIndex = 0;
            timer1.Interval = Properties.Settings.Default.StatusUpdateTiming;
            timer1.Start();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            UpdateRobotUI();
            UpdateJobList();
        }

        private void buttonDriveStats_Click(object sender, EventArgs e)
        {
            myStatusFile.Refresh();
            string sMsg = ""; 

            foreach (Drive myDrive in myStatusFile.Drives)
            {
                sMsg += myDrive.Description + "\n" + myDrive.GoodDiscs + " Good Discs\n" + myDrive.BadDiscs + " Bad Discs\n";
                sMsg += "   " + myDrive.DriveNotReadyErrors + " Drive Not Ready Errors\n";
                sMsg += "   " + myDrive.RecordingErrors + " Recording Errors\n";
                sMsg += "   " + myDrive.RecordingErrorsMedia + " Recording Errors (Media)\n";
                sMsg += "   " + myDrive.VerifyErrors + " Verifying Errors\n";
                sMsg += "   " + myDrive.VerifyErrorsMedia + " Verifying Errors (Media)\n";
                sMsg += "   " + myDrive.ReadErrors + " Reading Errors\n";
                sMsg += "   " + myDrive.ReadErrorsMedia + " Reading Errors (Media)\n\n";
            }
            MessageBox.Show(sMsg);
        }
    }
}
