using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Collections;
using PrimeraTechnology;

namespace PTRobotUserControl
{
    public partial class PTRobotUserControl1 : UserControl
    {
        private FormMergeData MyMergeForm;
        private FormPrinterSettings MyPrinterSettings;
        private uint[] RobotArray;
        private uint[] DriveArray;
        private uint NumRobots;
        private uint NumDrives;
        private string PrintFile;
        private uint PreviewResolution;
        private string PreviewFormat;

        public PTRobotUserControl1()
        {
            InitializeComponent();

            // handle up to 10 robots (each robot can have up to 2 drives)
            NumRobots = 10;                  
            NumDrives = 2;                    
            RobotArray = new uint[NumRobots];
            DriveArray = new uint[NumDrives];


            MyMergeForm = new FormMergeData();
            MyPrinterSettings = new FormPrinterSettings();
            // Setup our SureThing Preview settings
            PreviewResolution = 72;
            comboBoxOutputDPI.Items.Add(72);
            comboBoxOutputDPI.Items.Add(300);
            comboBoxOutputDPI.Items.Add(600);
            comboBoxOutputDPI.SelectedIndex = 0;
            PreviewFormat = "JPG";
            comboBoxPreviewFormat.Items.Add("JPG");
            comboBoxPreviewFormat.Items.Add("BMP");
            comboBoxPreviewFormat.Items.Add("PNM");
            comboBoxPreviewFormat.SelectedIndex = 0;


        }


        private bool GetAttachedRobots()
        {
            bool retVal = false;
            if (EnumRobots())
            {
                buttonGo.Enabled = true;
                retVal = true;
            }
            /*else
            {
                DialogResult Response = MessageBox.Show("Unable to enumerate Robots", "Error", MessageBoxButtons.RetryCancel);
                if (Response == DialogResult.Cancel)
                {
                    buttonGo.Enabled = false;
                    return false;
                }
            }*/
            return retVal;
        }

        
        /// <summary>
        /// EnumRobots()
        /// </summary>
        private bool EnumRobots()
        {
            bool retVal = false;
            uint numRobots = NumRobots;
            RobotInfo myRobotInfo = new RobotInfo();

            comboBoxRobots.Items.Clear();

            if (0 == PTRobot.EnumRobots(ref RobotArray[0], ref numRobots))
            {
                for (int i = 0; i<numRobots; i++)
                {
                    if (PTRobotReturn.Ok == PTRobot.GetRobotInfo(RobotArray[i], ref myRobotInfo))
                    {
                        comboBoxRobots.Items.Add(myRobotInfo.RobotDesc);
                        retVal = true;
                    }
                }

                if (retVal)
                    comboBoxRobots.SelectedIndex = 0;
            }

            return retVal;
        }


        private void ShowRobotInfo(int nIndex)
        {
            RobotInfo myRobotInfo = new RobotInfo();
            ManufactureInfo myManufactureInfo = new ManufactureInfo();

            if (nIndex >= 0)
            {
                if (PTRobotReturn.Ok == PTRobot.GetRobotInfo(RobotArray[nIndex], ref myRobotInfo))
                {
                    TextRobotType.Text = myRobotInfo.RobotType.ToString();
                    TextNumDrives.Text = myRobotInfo.NumDrives.ToString();
                    TextNumBins.Text = myRobotInfo.NumBins.ToString();
                }
                if (PTRobotReturn.Ok == PTRobot.GetManufactureInfo(RobotArray[nIndex], ref myManufactureInfo))
                {
                    textManufactureDate.Text = myManufactureInfo.ManufactureDate;
                    textSerialNum.Text = myManufactureInfo.SerialNum;
                }
                else
                {
                    textManufactureDate.Text = "---";
                    textSerialNum.Text = "---";
                }
                this.buttonMoreRobotInfo.Enabled = true;
                this.ButtonEnumDrives.Enabled = true;
                this.groupBox2.Enabled = true;
                this.groupBox3.Enabled = true;
                if (myRobotInfo.NumDrives <= 0)
                {
                    this.checkBoxRecorders.Enabled = false;
                    this.checkBoxOpenCloseCallback.Enabled = false;
                }
                else
                {
                    this.checkBoxRecorders.Enabled = true;
                    this.checkBoxOpenCloseCallback.Enabled = true;
                }
            }
            else
            {
                TextRobotType.Text = "";
                TextNumDrives.Text = "";
                TextNumBins.Text = "";
                textManufactureDate.Text = "";
                textSerialNum.Text = "";

                this.buttonMoreRobotInfo.Enabled = false;
                this.ButtonEnumDrives.Enabled = false;
                this.groupBox2.Enabled = false;
                this.groupBox3.Enabled = false;

            }
        }
        

        private bool ShowDriveInfo( uint nRobotID)
        {
            int index;
            bool retVal = false;
            uint numDrives = NumDrives;
            PrimeraTechnology.DriveInfo myDriveInfo = new PrimeraTechnology.DriveInfo();

            ListBoxDrives.Items.Clear();

            if (PTRobotReturn.Ok == PTRobot.EnumDrives(nRobotID, ref DriveArray[0], ref numDrives))
            {
                for (index = 0; index< numDrives; index++)
                {
                    if (PTRobotReturn.Ok == PTRobot.GetDriveInfo(DriveArray[index], ref myDriveInfo))
                    {
                        ListBoxDrives.Items.Add(myDriveInfo.DriveName);
                        this.ButtonDriveInfo.Enabled = true;
                        this.buttonMediaInfo.Enabled = true;
                        this.ButtonOpenClose.Enabled = true;
                    }
                    retVal = true;
                }
            }

            return retVal;
        }
        
        
        /// <summary>
        /// BrowsePrintButton_Click()
        /// 
        /// Retrieve the file to print (and show a preview).
        /// For SureThing files we must call PTRobot to generate a preview
        /// ** NOTE: we are not deleting the preview after it is created **
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BrowsePrintButton_Click(object sender, EventArgs e)
        {
            bool showPreview = false;

            // Configure open file dialog box
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.FileName = ""; // Default file name
            dlg.DefaultExt = ".std"; // Default file extension
            dlg.Filter = "Image Files(*.STD;*.BMP;*.JPG;*.PNG)|*.STD;*.BMP;*.JPG;*.PNG"; // Filter files by extension

            // Show open file dialog box
            DialogResult result = dlg.ShowDialog();

            // Process open file dialog box results
            if (result == DialogResult.OK)
            {
                // Open document
                PrintFile = dlg.FileName;
                string sPreviewFile = PrintFile;
                PrintFileTextBox.Text = sPreviewFile;
                this.buttonPrinterSettings.Enabled = true;

                if (sPreviewFile.ToLower().EndsWith(".std"))
                {
                    // Need to GENERATE a preview for SureThing files...
                    sPreviewFile = PrintFile + "." + PreviewFormat;
                    if (File.Exists(sPreviewFile))
                        File.Delete(sPreviewFile);
                    PTRobot.GetSureThingPreview(PrintFile, sPreviewFile, PreviewResolution);
                    showPreview = UtilityFunctions.UtilityFunctions.WaitUntilExists(sPreviewFile, 10000);
                    this.buttonSetMergeFields.Enabled = true;
                }
                else
                    this.buttonSetMergeFields.Enabled = false;

                if (showPreview)
                {
                    PrintImage.SizeMode = PictureBoxSizeMode.StretchImage;
                    PrintImage.ImageLocation = sPreviewFile;
                    PrintImage.Load();
                }
            }
        }






        private void comboBoxOutputDPI_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(comboBoxOutputDPI.Text))
                PreviewResolution = Convert.ToUInt32(comboBoxOutputDPI.Text);

        }

        private void comboBoxPreviewFormat_SelectedIndexChanged(object sender, EventArgs e)
        {
            PreviewFormat = comboBoxPreviewFormat.Text;
        }

        private void comboBoxRobots_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowRobotInfo(comboBoxRobots.SelectedIndex);
            ShowDriveInfo(RobotArray[comboBoxRobots.SelectedIndex]);
        }

        private void buttonEnumRobots_Click(object sender, EventArgs e)
        {

            if (GetAttachedRobots())
            {
                this.buttonMoreRobotInfo.Enabled = true;
                ShowRobotInfo(comboBoxRobots.SelectedIndex);
                ShowDriveInfo(RobotArray[comboBoxRobots.SelectedIndex]);
            }
            else
                this.buttonMoreRobotInfo.Enabled = false;
        }

        /// <summary>
        /// buttonGo_Click()
        ///
        /// GO BUTTON PRESSED - gather the settings and pass them on
        /// to the thread to perform the movements.
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonGo_Click(object sender, EventArgs e)
        {
            uint copies;
            ThreadSettings myThreadSettings = new ThreadSettings();

            myThreadSettings.Copies = (uint)numericUpDown1.Value;

            // Set drive OPEN/CLOSE callback function (if requested)
            if (checkBoxOpenCloseCallback.Checked)
                PTRobot.SetOpenCloseCallback(null);        // todo: add callback function
            else
                PTRobot.SetOpenCloseCallback(null);

            for (int i=0; i<NumDrives; i++)
                myThreadSettings.DriveArray[i] = DriveArray[i];

            myThreadSettings.UseRecorders = checkBoxRecorders.Checked;
            myThreadSettings.UsePrinter = checkBoxPrinter.Checked;
            Array values = Enum.GetValues(typeof(PrimeraTechnology.DiscLocation));
            myThreadSettings.ToLocation = (PrimeraTechnology.DiscLocation)values.GetValue(comboBoxToLocation.SelectedIndex);
            myThreadSettings.FromLocation = (PrimeraTechnology.DiscLocation)values.GetValue(comboBoxFromLocation.SelectedIndex);


            myThreadSettings.RobotID = RobotArray[comboBoxRobots.SelectedIndex];
            myThreadSettings.PrintFile = PrintFile;
            FormRunTest myTestRunner = new FormRunTest(myThreadSettings);
            myTestRunner.ShowDialog();


            //m_PTRobotDLL.PrintFile(m_nRobotArray[0], m_sPrintFile, 0);
        }



        private void ButtonOpenClose_Click(object sender, EventArgs e)
        {
            PTRobot.OpenCloseDrive(DriveArray[0], 0);
        }

        private void PTRobotUserControl1_Load(object sender, EventArgs e)
        {
            // Setup debugging and INIT the DLL
            PTRobot.SetupDebugging(@"c:\PTRobotDbg.txt", 4, @"c:\PTRobotDbg.txt");
            PTRobot.Initialize();
            this.ButtonDriveInfo.Enabled = false;
            this.buttonMediaInfo.Enabled = false;
            this.ButtonOpenClose.Enabled = false;

            PTRobot.SetOpenCloseCallback(null);    // TODO: need to add callback
            // Emumerate all attached robots.  Select and show the first one.
            if (GetAttachedRobots())
            {
                comboBoxRobots.SelectedIndex = 0;
                ShowRobotInfo(comboBoxRobots.SelectedIndex);
                ShowDriveInfo(RobotArray[comboBoxRobots.SelectedIndex]);
            }

            foreach (PrimeraTechnology.DiscLocation discLoc in Enum.GetValues(typeof(PrimeraTechnology.DiscLocation)))
            {
                this.comboBoxToLocation.Items.Add(discLoc.ToString());
                this.comboBoxFromLocation.Items.Add(discLoc.ToString());
            }
            this.comboBoxFromLocation.SelectedIndex = 0;
            this.comboBoxToLocation.SelectedIndex = 0;


            this.buttonSetMergeFields.Enabled = false;
            this.buttonPrinterSettings.Enabled = false;
        }

        private void SetupForRobot()
        {
            RobotInfo myRobotInfo = null;
            UInt32 robotID = RobotArray[comboBoxRobots.SelectedIndex];
            if (PTRobotReturn.Ok == PTRobot.GetRobotInfo(robotID, ref myRobotInfo))
            {
                this.comboBoxFromLocation.Items.Clear();
                this.comboBoxToLocation.Items.Clear();
            }

        }

        private void ButtonDriveInfo_Click(object sender, EventArgs e)
        {
            int index = 0;
            PrimeraTechnology.DriveInfo myDriveInfo = new PrimeraTechnology.DriveInfo();

            if (this.ListBoxDrives.SelectedIndex > 0)
                index = this.ListBoxDrives.SelectedIndex;

            UInt32 driveID = DriveArray[index];

            PTRobot.GetDriveInfo(driveID, ref myDriveInfo);

            string msg = myDriveInfo.DriveName + "\n   Firmware: " + myDriveInfo.FirmwareVer + 
                "\n   Serial: " + myDriveInfo.SerialNum;

            MessageBox.Show(msg);
        }

        private void buttonMoreRobotInfo_Click(object sender, EventArgs e)
        {
            RobotInfo myRobotInfo = new RobotInfo();
            RobotInfo2 myRobot2Info = new RobotInfo2();

            PTRobot.GetRobotInfo(RobotArray[comboBoxRobots.SelectedIndex], ref myRobotInfo);
            PTRobot.GetRobotInfo2(RobotArray[comboBoxRobots.SelectedIndex], ref myRobot2Info);

            string msg = "FW: " + myRobotInfo.RobotFirmware + "\n" +
                          "FW2 Code: " + myRobot2Info.Firmware2Code.ToString() + "\n" +
                          "PGA Ver: " + myRobot2Info.PGA.ToString() + "\n" +
                          "USB Serial: " + myRobot2Info.USBSerialNum.ToString() + "\n" +
                          "Options: " + myRobotInfo.SupportedOptions.ToString() + "\n" +
                          "Actions: " + myRobotInfo.SupportedActions.ToString() + "\n" +
                          "BusType : " + myRobotInfo.DriveBusType.ToString() + "\n";
            for (int i = 0; i < myRobot2Info.NumCartridges; i++)
            {
                msg += "Cart: " + i.ToString() + myRobot2Info.CartridgeType[i].ToString() +"\n";
            }
            MessageBox.Show(msg);


        }

        private void groupBox3_Enter(object sender, EventArgs e)
        {

        }

        private void buttonSetMergeFields_Click(object sender, EventArgs e)
        {
            MyMergeForm.ShowDialog();
        }

        private void buttonPrinterSettings_Click(object sender, EventArgs e)
        {
            this.MyPrinterSettings.ShowDialog();
        }

        private void buttonMediaInfo_Click(object sender, EventArgs e)
        {
            PrimeraTechnology.MediaInfo myMedia = new MediaInfo();
            int index = 0;

            if (this.ListBoxDrives.SelectedIndex > 0)
                index = this.ListBoxDrives.SelectedIndex;

            UInt32 dwDriveID = DriveArray[index];

            PTRobotReturn result = PTRobot.GetMediaInfo(dwDriveID, ref myMedia);

            string msg;
            if (result == PTRobotReturn.Ok)
            {
                msg = "Media in Drive\n   Media Type: " + myMedia.MediaType +
                    "\n   Media ID: " + myMedia.MediaID;
            }
            else if (result == PTRobotReturn.InvalidMedia)
                msg = "Invalid Media";
            else if (result == PTRobotReturn.NoMedia)
                msg = "No Media";
            else
                msg = "Error Getting Media Info " + result.ToString();


            MessageBox.Show(msg);

        }

        private void groupBox3_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void groupBox3_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);


            // loop through the string array, adding each filename to the ListBox
            foreach (string file in files)
            {
                this.PrintFileTextBox.Text = file;
                break;
                //this.labelDragFiles.Visible = false;
            }
        }

        private void PrintFileTextBox_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);


            // loop through the string array, adding each filename to the ListBox
            foreach (string file in files)
            {
                this.PrintFileTextBox.Text = file;
                break;
                //this.labelDragFiles.Visible = false;
            }
        }

        private void PrintFileTextBox_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }
    }
}
