using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;

namespace SampleClient
{
    public enum CartridgeType {NO_CARTRIDGE=0, COLOR_CARTRIDGE, BLACK_CARTRIDGE, YELLOW_CARTRIDGE, CYAN_CARTRIDGE, MAGENTA_CARTRIDGE };
    
    public class Robot
    {
        private string RobotName, File, Path;
        public UInt32 NumDrives;
        public RobotType RobotType;
        public string StatusFile;
        public UInt32 NumJobs;
        public string Status;
        public SystemError Error;
        public UInt32 DiscsInBin0;
        public UInt32 DiscsInBin1;
        public CartridgeType CartType0;
        public CartridgeType CartType1;
        public CartridgeType CartType2;
        public CartridgeType CartType3;
        public UInt32 CartFill0;
        public UInt32 CartFill1;
        public UInt32 CartFill2;
        public UInt32 CartFill3;
        public string Name;


        public Robot(string robotIn, string pathIn, string fileIn)
        {
            RobotName = Name = robotIn;
            Path = pathIn;
            File = fileIn;
            this.Refresh();


        }
        public void Refresh()
        {

            string temp;

            temp = Utility.INIGetKeyValue(RobotName, "SystemDrives", File);
            if (!String.IsNullOrEmpty(temp))
                this.NumDrives = Convert.ToUInt32(temp);

            temp = Utility.INIGetKeyValue(RobotName, "RobotType", File);
            if (!String.IsNullOrEmpty(temp))
                this.RobotType = (RobotType)(Convert.ToUInt32(temp));

            temp = Utility.INIGetKeyValue(RobotName, "StatusFile", File);
            if (!String.IsNullOrEmpty(temp))
                this.StatusFile = temp;

            temp = Utility.INIGetKeyValue(RobotName, "NumJobs", File);
            if (!String.IsNullOrEmpty(temp))
                this.NumJobs = Convert.ToUInt32(temp);

            temp = Utility.INIGetKeyValue(RobotName, "SysErrorNumber", File);
            if (!String.IsNullOrEmpty(temp))
                this.Error = (SystemError)Convert.ToUInt32(temp);


            temp = Utility.INIGetKeyValue(RobotName, "CartridgeType0", File);
            if (!String.IsNullOrEmpty(temp))
            {
                try
                {
                    this.CartType0 = (CartridgeType)Convert.ToUInt32(temp);
                }
                catch
                {
                    this.CartType0 = (CartridgeType)0;
                }
            }
            temp = Utility.INIGetKeyValue(RobotName, "CartridgeFill0", File);
            if (!String.IsNullOrEmpty(temp))
                this.CartFill0 = Convert.ToUInt32(temp);

            
            // Is this a robot with  individual 4-color cartridges?
            // If so, note that the format in the INI file (for backwards compatibility) will be:  BLACK, COLOR, YELLOW, CYAN, MAGENTA
            // So we will ignore the more generic COLOR value and just get the individual colors
            bool fIndividualTanks = false;
            temp = Utility.INIGetKeyValue(RobotName, "CartridgeType4", File);
            if (!String.IsNullOrEmpty(temp))
            {
                fIndividualTanks = true;
                this.CartType3 = (CartridgeType)Convert.ToUInt32(temp);
                temp = Utility.INIGetKeyValue(RobotName, "CartridgeFill4", File);
                if (!String.IsNullOrEmpty(temp))
                    this.CartFill3 = Convert.ToUInt32(temp);
            }

            if (fIndividualTanks)
            {
                ///////////////////////////////////////////////////////
                // INDIVIDUAL COLOR CARTRIDGES - yellow, cyan, magenta
                ///////////////////////////////////////////////////////
                temp = Utility.INIGetKeyValue(RobotName, "CartridgeType3", File);
                if (!String.IsNullOrEmpty(temp))
                {
                    fIndividualTanks = true;
                    this.CartType2 = (CartridgeType)Convert.ToUInt32(temp);
                    temp = Utility.INIGetKeyValue(RobotName, "CartridgeFill3", File);
                    if (!String.IsNullOrEmpty(temp))
                        this.CartFill2 = Convert.ToUInt32(temp);
                }

                temp = Utility.INIGetKeyValue(RobotName, "CartridgeType2", File);
                if (!String.IsNullOrEmpty(temp))
                {
                    fIndividualTanks = true;
                    this.CartType1 = (CartridgeType)Convert.ToUInt32(temp);
                    temp = Utility.INIGetKeyValue(RobotName, "CartridgeFill2", File);
                    if (!String.IsNullOrEmpty(temp))
                        this.CartFill1 = Convert.ToUInt32(temp);
                }
            }
            else
            {
                /////////////////////////////////////////
                // COLOR CARTRIDGE (not individual tanks)
                /////////////////////////////////////////
                temp = Utility.INIGetKeyValue(RobotName, "CartridgeType1", File);
                if (!String.IsNullOrEmpty(temp))
                {
                    try
                    {
                        this.CartType1 = (CartridgeType)Convert.ToUInt32(temp);
                    }
                    catch
                    {
                        this.CartType1 = (CartridgeType)0;
                    }
                }
                temp = Utility.INIGetKeyValue(RobotName, "CartridgeFill1", File);
                if (!String.IsNullOrEmpty(temp))
                    this.CartFill1 = Convert.ToUInt32(temp);
            }




            temp = Utility.INIGetKeyValue(RobotName, "DiscsInLeftBin", File);
            if (!String.IsNullOrEmpty(temp))
            {
                try
                {
                    this.DiscsInBin0 = Convert.ToUInt32(temp);
                }
                catch
                {
                    this.DiscsInBin0 = 255;
                }
            }
            else
                this.DiscsInBin0 = 9999;

            temp = Utility.INIGetKeyValue(RobotName, "DiscsInRightBin", File);
            if (!String.IsNullOrEmpty(temp))
            {
                try
                {
                    this.DiscsInBin1 = Convert.ToUInt32(temp);
                }
                catch
                {
                    this.DiscsInBin1 = 255;
                }
            }
            else
                this.DiscsInBin1 = 9999;


            // Get Status from robot Status file
            temp = Utility.INIGetKeyValue("System", "SystemStatus", Path + this.StatusFile);
            if (!String.IsNullOrEmpty(temp))
                this.Status = temp;


        }

    }
    
    
    class SystemStatus
    {
        public ArrayList Robots;
        private string path;
        private string file;
        
        public SystemStatus(string fileIn)
        {
            Robots = new ArrayList();
            path = fileIn;
            file = path + "SystemStatus.txt";  
        }

        public void Refresh()
        {
            int robotIndex = 0;
            

            if (File.Exists(file))
            {
                while (true)
                {
                    string sRobotKey = "Robot" + robotIndex.ToString();
                    string sTemp = Utility.INIGetKeyValue("RobotList", sRobotKey, file);
                    if (!String.IsNullOrEmpty(sTemp))
                        this.Robots.Add(new Robot(sTemp, path, file));
                    else
                        break;
                    robotIndex++;
                }
            }
        }
    }
}
